/* TickerTape/89 1.0
 * (c) 2001 Sam Stephenson
 * sam@conio.net - www.conio.net
 * 
 * This program is freely distributable under
 * the terms of a BSD-style license.  Please
 * see LICENSE.TXT for further information.
 */

#define OPTIMIZE_ROM_CALLS
#define SAVE_SCREEN

#include <alloc.h>
#include <args.h>
#include <dialogs.h>
#include <error.h>
#include <kbd.h>
#include <string.h>
#include <system.h>
#include <wingraph.h>

short _ti89;

void showAbout (void) {
	DlgMessage ("About", 
		"TickerTape/89 v1.0\n(c) 2001 Sam Stephenson\nsam@conio.net\
 \xb7 www.conio.net\n", BT_NONE, BT_OK);
	return;
}

void _main (void)
{
	ESI argptr;
	WINDOW wndMain;
	const int ypos=42;
	int x,y,key,run,speed;
	char *text,*buf;
	void *kbq = kbd_queue ();
	
	/* first, get the string that was passed (if any) */
	InitArgPtr (argptr);
	switch (GetArgType (argptr)) {
		case END_TAG:
			/* no string was specified; use the default */
			if ((text = calloc (25, sizeof (char))) == NULL) {
				ERD_dialog (670, FALSE);
				return;
			} else {
				strcpy (text, "Welcome to TickerTape/89 ");
				break;
			}			
			break;
		case STR_TAG:
			/* allocate space and copy into text */
			buf = GetStrnArg (argptr);
			if ((text = calloc (strlen (buf) + 1,
				sizeof (char))) == NULL) {
					ERD_dialog (670, FALSE);
					return;
			} else {
				strcpy (text, buf);
				text[strlen (buf)] = ' ';
				text[strlen (buf) + 1] = '\0';
				break;
			}
			break;
		default:
			/* invalid type; give error and quit */
			ERD_dialog (130, FALSE);
			return;
	}
	
	/* next, get the speed that was passed (if any) */
	switch (GetArgType (argptr)) {
		case END_TAG:
			speed = 3;
			break;
		case POSINT_TAG:
		case NEGINT_TAG:
			speed = GetIntArg (argptr);
			if (speed > 5) speed = 5;
			if (speed < 1) speed = 1;
			break;
		default:
			/* it's supposed to be an integer, stupid */
			ERD_dialog (60, FALSE);
			return;
	}
			
	
	/* create the window */
	WinOpen (&wndMain, 
		MakeWinRect (0, ypos, 160, ypos+16),
		WF_SAVE_SCR | WF_NOBOLD);
	WinActivate (&wndMain);
	WinFont (&wndMain, F_8x10);
	WinAttr (&wndMain, A_REPLACE);


	/* set up initial positions and calculate length */
	x = 160;
	y = strlen(text) * 8;
	run = 1;
		
	/* initialize the timer */
	OSFreeTimer (USER_TIMER);
	OSRegisterTimer (USER_TIMER, 2);

	/* go! */
	while (run) {
		if (!OSdequeue (&key, kbq)) {
			/* handle keypress */
			switch (key) {
				case KEY_ESC:
					/* quit */
					run = 0;
					break;
				case KEY_ENTER:
					/* pause */
					idle ();
					if (ngetchx () == KEY_ESC)
						run = 0;
					break;
				case '+':
					/* faster */
					speed++;
					if (speed > 5) speed = 5;
					break;
				case '-':
					/* slower */
					speed--;
					if (speed < 1) speed = 1;
					break;
				case 268:
					/* F1 = about */
					showAbout ();
					break;
				default:
					break;
			}
		}
		
		if (OSTimerExpired (USER_TIMER)) {
			OSTimerRestart (USER_TIMER);
			WinStrXY (&wndMain, x, 2, text);
			if ((x -= speed) < -y) x = 160;
		}
		
		idle ();
	}

	/* exit cleanly */
	OSFreeTimer (USER_TIMER);
	WinClose (&wndMain);
	free (text);
}
